@@ -5,7 +5,7 @@
 
    Copyright (C) 1997-2005 Luke Howard
    Copyright (C) 2007 West Consulting
-   Copyright (C) 2007-2017 Arthur de Jong
+   Copyright (C) 2007-2018 Arthur de Jong
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -425,7 +425,7 @@ static void add_uris_from_dns(const char *filename, int lnr,
 {
   int rc;
   char *hostlist = NULL, *nxt;
-  char buf[HOST_NAME_MAX + sizeof("ldap://")];
+  char buf[BUFLEN_HOSTNAME + sizeof("ldap://")];
   log_log(LOG_DEBUG, "query %s for SRV records", domain);
   rc = ldap_domain2hostlist(domain, &hostlist);
   if (rc != LDAP_SUCCESS)
@@ -802,7 +802,7 @@ static void handle_map(const char *filename, int lnr,
 {
   enum ldap_map_selector map;
   const char **var;
-  char oldatt[32], newatt[1024];
+  char oldatt[32], *newatt;
   /* get the map */
   if ((map = get_map(&line)) == LM_NONE)
   {
@@ -811,10 +811,8 @@ static void handle_map(const char *filename, int lnr,
   }
   /* read the other tokens */
   check_argumentcount(filename, lnr, keyword,
-                      (get_token(&line, oldatt, sizeof(oldatt)) != NULL) &&
-                      (get_token(&line, newatt, sizeof(newatt)) != NULL));
-  /* check that there are no more tokens left on the line */
-  get_eol(filename, lnr, keyword, &line);
+                      (get_token(&line, oldatt, sizeof(oldatt)) != NULL));
+  newatt = get_linedup(filename, lnr, keyword, &line);
   /* change attribute mapping */
   var = attmap_get_var(map, oldatt);
   if (var == NULL)
@@ -829,6 +827,7 @@ static void handle_map(const char *filename, int lnr,
             filename, lnr, oldatt);
     exit(EXIT_FAILURE);
   }
+  free(newatt);
 }
 
 #ifdef LDAP_OPT_X_TLS
@@ -1266,7 +1265,7 @@ static void cfg_read(const char *filename, struct ldap_config *cfg)
   char linebuf[MAX_LINE_LENGTH];
   char *line;
   char keyword[32];
-  char token[64];
+  char token[256];
   int i;
 #ifdef LDAP_OPT_X_TLS
   int rc;
