@@ -5,7 +5,7 @@
 
    Copyright (C) 1997-2005 Luke Howard
    Copyright (C) 2006 West Consulting
-   Copyright (C) 2006-2014 Arthur de Jong
+   Copyright (C) 2006-2017 Arthur de Jong
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -103,6 +103,7 @@ static int mkfilter_passwd_byname(const char *name,
    by uid, return -1 on errors */
 static int mkfilter_passwd_byuid(uid_t uid, char *buffer, size_t buflen)
 {
+  uid -= nslcd_cfg->nss_uid_offset;
   if (uidSid != NULL)
   {
     return mysnprintf(buffer, buflen, "(&%s(%s=%s\\%02x\\%02x\\%02x\\%02x))",
@@ -207,7 +208,12 @@ static int entry_has_valid_uid(MYLDAP_ENTRY *entry)
         continue;
       }
     }
-    if (uid >= nslcd_cfg->nss_min_uid)
+    if (uid < nslcd_cfg->nss_min_uid)
+    {
+      log_log(LOG_DEBUG, "%s: %s: less than nss_min_uid",
+              myldap_get_dn(entry), attmap_passwd_uidNumber);
+    }
+    else
       return 1;
   }
   /* nothing found */
@@ -292,7 +298,7 @@ char *dn2uid(MYLDAP_SESSION *session, const char *dn, char *buf, size_t buflen)
   {
     if ((cacheentry->uid != NULL) && (strlen(cacheentry->uid) < buflen))
     {
-      /* if the cached entry is still valid, return that */
+      /* positive hit: if the cached entry is still valid, return that */
       if ((nslcd_cfg->cache_dn2uid_positive > 0) &&
           (time(NULL) < (cacheentry->timestamp + nslcd_cfg->cache_dn2uid_positive)))
       {
@@ -303,9 +309,9 @@ char *dn2uid(MYLDAP_SESSION *session, const char *dn, char *buf, size_t buflen)
     }
     else
     {
+      /* negative hit: if the cached entry is still valid, return that */
       if ((nslcd_cfg->cache_dn2uid_negative > 0) &&
            (time(NULL) < (cacheentry->timestamp + nslcd_cfg->cache_dn2uid_negative)))
-      /* if the cached entry is still valid, return that */
       {
         pthread_mutex_unlock(&dn2uid_cache_mutex);
         return NULL;
@@ -481,6 +487,12 @@ static int write_passwd(TFILE *fp, MYLDAP_ENTRY *entry, const char *requser,
           return 0;
         }
       }
+      uids[numuids] += nslcd_cfg->nss_uid_offset;
+      if (uids[numuids] < nslcd_cfg->nss_min_uid)
+      {
+          log_log(LOG_DEBUG, "%s: %s: less than nss_min_uid",
+                  myldap_get_dn(entry), attmap_passwd_uidNumber);
+      }
     }
   }
   /* get the gid for this entry */
@@ -519,6 +531,7 @@ static int write_passwd(TFILE *fp, MYLDAP_ENTRY *entry, const char *requser,
       return 0;
     }
   }
+  gid += nslcd_cfg->nss_gid_offset;
   /* get the gecos for this entry */
   attmap_get_value(entry, attmap_passwd_gecos, gecos, sizeof(gecos));
   /* get the home directory for this entry */
@@ -530,6 +543,7 @@ static int write_passwd(TFILE *fp, MYLDAP_ENTRY *entry, const char *requser,
   attmap_get_value(entry, attmap_passwd_loginShell, shell, sizeof(shell));
   /* write the entries */
   for (i = 0; usernames[i] != NULL; i++)
+  {
     if ((requser == NULL) || (STR_CMP(requser, usernames[i]) == 0))
     {
       if (!isvalidname(usernames[i]))
@@ -555,6 +569,7 @@ static int write_passwd(TFILE *fp, MYLDAP_ENTRY *entry, const char *requser,
         }
       }
     }
+  }
   return 0;
 }
 
@@ -582,6 +597,7 @@ NSLCD_HANDLE_UID(
   log_setrequest("passwd=%lu", (unsigned long int)uid);
   if (uid < nslcd_cfg->nss_min_uid)
   {
+    log_log(LOG_DEBUG, "request ignored by nss_min_uid option");
     /* return an empty result */
     WRITE_INT32(fp, NSLCD_VERSION);
     WRITE_INT32(fp, NSLCD_ACTION_PASSWD_BYUID);
