@@ -1,7 +1,7 @@
 
 # tio.py - I/O functions
 #
-# Copyright (C) 2010, 2011 Arthur de Jong
+# Copyright (C) 2010, 2011, 2012, 2013 Arthur de Jong
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -18,21 +18,15 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 # 02110-1301 USA
 
-import struct
 import os
 import socket
+import struct
 
 
 # definition for reading and writing INT32 values
-_int32 = struct.Struct('i')
+_int32 = struct.Struct('!i')
 
-# FIXME: use something from config.py to determine the correct size
-_uid_t = struct.Struct('i')
-
-# FIXME: use something from config.py to determine the correct size
-_gid_t = struct.Struct('i')
-
-# FIXME: use something from config.py to determine the correct size
+# FIXME: use something from constants.py to determine the correct size
 _struct_timeval = struct.Struct('ll')
 
 
@@ -56,17 +50,11 @@ class TIOStream(object):
     def read_int32(self):
         return _int32.unpack(self.read(_int32.size))[0]
 
-    def read_uid_t(self):
-        return _uid_t.unpack(self.read(_uid_t.size))[0]
-
-    def read_gid_t(self):
-        return _gid_t.unpack(self.read(_gid_t.size))[0]
-
     def read_string(self, maxsize=None):
-        len = self.read_int32()
-        if maxsize and len >= maxsize:
+        num = self.read_int32()
+        if maxsize and num >= maxsize:
             raise TIOStreamError()
-        return self.read(len)
+        return self.read(num)
 
     def read_address(self):
         """Read an address (usually IPv4 or IPv6) from the stream and return
@@ -80,12 +68,6 @@ class TIOStream(object):
     def write_int32(self, value):
         self.write(_int32.pack(value))
 
-    def write_uid_t(self, value):
-        self.write(_uid_t.pack(value))
-
-    def write_gid_t(self, value):
-        self.write(_gid_t.pack(value))
-
     def write_string(self, value):
         self.write_int32(len(value))
         self.write(value)
