@@ -2,7 +2,7 @@
 
 # cmdline.py - functions for handling command-line options
 #
-# Copyright (C) 2013-2016 Arthur de Jong
+# Copyright (C) 2013-2017 Arthur de Jong
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -28,7 +28,7 @@ version_string = '''
 %s
 Written by Arthur de Jong.
 
-Copyright (C) 2013-2016 Arthur de Jong
+Copyright (C) 2013-2017 Arthur de Jong
 This is free software; see the source for copying conditions.  There is NO
 warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 '''.strip() % constants.PACKAGE_STRING
@@ -46,7 +46,7 @@ class VersionAction(argparse.Action):
             help=help)
 
     def __call__(self, parser, namespace, values, option_string=None):
-        print version_string
+        print(version_string)
         parser.exit()
 
 
@@ -64,5 +64,5 @@ class ListShellsAction(argparse.Action):
     def __call__(self, parser, namespace, values, option_string=None):
         import shells
         for shell in shells.list_shells():
-            print shell
+            print(shell)
         parser.exit()
